<?php

namespace Drupal\purest_menus\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfo;
use Drupal\Core\Entity\EntityFieldManager;

/**
 * Class ConfigForm.
 */
class NormalizerSettingsForm extends ConfigFormBase {

  /**
   * Drupal\Core\State\StateInterface definition.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * Drupal\Core\Config\ConfigFactoryInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The product storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $entityStorage;

  /**
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManager
   */
  protected $entityTypeBundle;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManager
   */
  protected $entityFieldManager;

  /**
   * 
   */
  public function __construct(ConfigFactoryInterface $config_factory, StateInterface $state, EntityTypeManagerInterface $entity_type_manager, EntityTypeBundleInfo $entity_type_bundle, EntityFieldManager $entity_field_manager) {
    parent::__construct($config_factory, $state);
    $this->configFactory = $config_factory;
    $this->state = $state;
    $this->entityTypeManager = $entity_type_manager;
    $this->entityFieldManager = $entity_field_manager;
    $this->entityTypeBundle = $entity_type_bundle;

    $this->entity_types = $this->entityTypeBundle->getBundleInfo('menu');
    $this->fields = $this->entityFieldManager->getFieldDefinitions('menu_link_content', 'content');
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('state'),
      $container->get('entity_type.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('entity_field.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'purest_menus.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'purest_menus_normalizer_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->configFactory->get('purest_menus.settings');
    
    $form['menus'] = [
      '#type' => 'field_group',
      '#title' => 'Menu Normalizer',
    ];

    $form['menus']['flatten'] = [
      '#type' => 'checkbox',
      '#title' => 'Flatten Menu Item Output',
      '#description' => $this->t('Drupal renders menu items in an overly complex structure with link items being a sub value. Turn this on to simplify the structure and place all attributes at the same level'),
      '#default_value' => NULL !== $config->get('flatten') ? 
                               1 :  $config->get('flatten'),
      '#suffix' => '<br>2'
    ];

    $form['menus']['menu_item_fields'] = [
      '#title' => $this->t('Menu Item Fields'),
      '#type' => 'table',
      '#sticky' => TRUE,
      '#header' => [
        $this->t('Name'),
        $this->t('Label'),
        $this->t('Type'),
        $this->t('Custom Label'),
        $this->t('Exclude')
      ],
    ];

    $values = $config->get('menu_item_fields');

    foreach ($this->fields as $field_name => $field_definition) {
      $form['menus']['menu_item_fields'][$field_name]['name'] = [
        '#type' => 'html_tag',
        '#tag' => 'span',
        '#value' => $field_definition->getLabel()
      ];

      $form['menus']['menu_item_fields'][$field_name]['label'] = [
        '#type' => 'html_tag',
        '#tag' => 'span',
        '#value' => $field_name
      ];

      $form['menus']['menu_item_fields'][$field_name]['type'] = [
        '#type' => 'textfield',
        '#disabled' => TRUE,
        '#size' => 30,
        '#default_value' => $field_definition->getType()
      ];

      $form['menus']['menu_item_fields'][$field_name]['custom_label'] = [
        '#type' => 'textfield',
        '#size' => 30,
        '#default_value' => NULL !== $values[$field_name]['custom_label'] ?
                                $values[$field_name]['custom_label'] : ''
      ];

      $form['menus']['menu_item_fields'][$field_name]['exclude'] = [
        '#type' => 'checkbox',
        '#default_value' => NULL !== $values[$field_name]['exclude'] ? 
                                $values[$field_name]['exclude'] : 0
      ];
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    dpm($form_state->getValue('menu_item_fields'));
    $config = $this->configFactory->getEditable('purest_menus.settings');
    $config->set('menu_item_fields', $form_state->getValue('menu_item_fields'));
    $config->save();
  }

}
