<?php

namespace Drupal\purest_user\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Class ConfigForm.
 */
class ConfigForm extends ConfigFormBase {

  /**
   * Drupal\Core\State\StateInterface definition.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * Drupal\Core\Config\ConfigFactoryInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The product storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $entityStorage;

  /**
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new RecaptchaConfigForm object.
   */
  public function __construct(ConfigFactoryInterface $config_factory, StateInterface $state, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($config_factory, $state);
    $this->configFactory = $config_factory;
    $this->state = $state;
    $this->entityTypeManager = $entity_type_manager;
    $this->pages = ['register', 'login', 'activate', 'reset'];
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('state'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'purest_user.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'purest_user_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->configFactory->get('purest_user.settings');
    $this->entityStorage = $this->entityTypeManager->getStorage('node');

    $form['#prefix'] = '<p>' . $this->t('Set nodes or custom paths to use for various user operations. These will be used in user emails to build URLs for account based actions.') . '</p>';

    $actions = ['register', 'login', 'activate', 'reset'];

    $form['user_pages'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('User Action Page Links')
    ];

    $form['user_pages']['user_actions'] = [
      '#type' => 'horizontal_tabs',
      '#default_tab' => 'edit-' . $actions[0],
    ];

    foreach ($this->pages as $action ) {
      $form['user_pages'][$action] = [
        '#type' => 'details',
        '#title' => $this->t(ucfirst($action)),
        '#group' => 'information'
      ];

      $form['user_pages'][$action][$action . '_node'] = [
        '#type' => 'entity_autocomplete',
        '#target_type' => 'node',
        '#title' => $this->t('Node'),
        '#description' => $this->t('A node to extract the path from. This will take precedence over any custom path below.'),
      ];

      $node_id = $config->get($action . '_node');
      
      if ($node_id) {
        $entity = $this->entityStorage->load($node_id);
        $form[$action][$action . '_node']['#default_value'] = $entity;
      }
      
      $form['user_pages'][$action][$action . '_path'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Custom Path'),
        '#description' => $this->t('A custom path for use if no node available.'),
        '#maxlength' => 256,
        '#size' => 64,
        '#default_value' => $config->get($action . '_path'),
      ];
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $config = $this->configFactory->getEditable('purest_user.settings');

    foreach ($this->pages as $action) {
      $config
      ->set($action . '_node', $form_state->getValue($action . '_node'))
      ->set($action . '_path', $form_state->getValue($action . '_path'));
    }

    $config->save();
  }

}
