<?php

namespace Drupal\purest_user\Plugin\rest\resource;

use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Drupal\Core\Session\AccountProxyInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Url;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Path\AliasManager;


/**
 * Creates a resource for retrieving current users profile
 *
 * @RestResource(
 *   id = "purest_user_profile_resource",
 *   label = @Translation("Purest User Profile"),
 *   uri_paths = {
 *     "canonical" = "/purest/user/profile"
 *   }
 * )
 */
class ProfileResource extends ResourceBase {
  /**
   * The current user instance.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $currentRequest;

  /**
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Entity storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $userStorage;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('rest'),
      $container->get('current_user'),
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('entity_type.manager')
    );
  }

  /**
   * Constructs a Drupal\rest\Plugin\ResourceBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param array $serializer_formats
   *   The available serialization formats.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, array $serializer_formats, LoggerInterface $logger, AccountProxyInterface $current_user, Request $current_request, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);

    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $current_user;
    $this->currentRequest = $current_request;
    $this->userStorage = $this->entityTypeManager->getStorage('user');
  }

  /**
   * Responds to GET requests and returns the order.
   * 
   * @return \Drupal\rest\ResourceResponse
   *   The order
   */
  public function get() {
    $user = $this->currentUser->getAccount();

    if($user){
      // Check if the loaded order is owned by the logged in user, if not return a 404.
      $response = new ResourceResponse($user, 200);
      $response->addCacheableDependency($this->currentUser);

      return $response;
    }
    else {
      return new ResourceResponse(['error' => 'User not found.'], 404);
    }
  }

}
