<?php

namespace Drupal\purest_user\Plugin\rest\resource;

use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\rest\ModifiedResourceResponse;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Drupal\purest_user\AccountValidationServiceInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Drupal\purest_recaptcha\RecaptchaInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides a resource to get view modes by entity and bundle.
 *
 * @RestResource(
 *   id = "purest_user_reset_resource",
 *   label = @Translation("Purest User Reset"),
 *   uri_paths = {
 *     "canonical" = "/purest/user/reset"
 *   }
 * )
 */
class ResetResource extends ResourceBase {

  /**
   * A current user instance.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The account validation service
   */
  protected $accountValidationService;

  /**
   * 
   */
  protected $entityTypeManager;

  /**
   * reCAPTCHA service
   */
  protected $recaptcha;

  /**
   * The reCAPTCHA response
   */
  protected $recaptchaResponse;

  /**
   * Constructs a new UserActivationRestResource object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param array $serializer_formats
   *   The available serialization formats.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   A current user instance.
   */
  public function __construct( array $configuration, $plugin_id, $plugin_definition, array $serializer_formats, LoggerInterface $logger, AccountProxyInterface $current_user, AccountValidationServiceInterface $validation_service, EntityTypeManagerInterface $entity_type_manager, RecaptchaInterface $recaptcha, Request $current_request) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);

    $this->currentUser = $current_user;
    $this->accountValidationService = $validation_service;
    $this->entityTypeManager = $entity_type_manager;
    $this->recaptcha = $recaptcha;
    $this->currentRequest = $current_request;
    $this->recaptchaResponse = $this->currentRequest->query->get('recaptcha');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('purest_user'),
      $container->get('current_user'),
      $container->get('purest_user.validation'),
      $container->get('entity_type.manager'),
      $container->get('purest_recaptcha.recaptcha'),
      $container->get('request_stack')->getCurrentRequest()
    );
  }

  /**
   * Responds to PATCH requests.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity object.
   *
   * @return \Drupal\rest\ModifiedResourceResponse
   *   The HTTP response object.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   *   Throws exception expected.
   */
  public function patch($data) {

    if(!isset($data['id']) || !isset($data['token']) || !isset($data['timestamp'])) {
      throw new BadRequestHttpException("id, token and timestamp must be provided."); 
    }

    $user_storage = $this->entityTypeManager->getStorage('user');
    $user = $user_storage->load($data['id']);
    if(!$user) {
      throw new BadRequestHttpException("User does not exist."); 
    }

    $active = $this->accountValidationService->activateAccount($user, $data['token'], $data['timestamp']);

    if($active) {
      return new ModifiedResourceResponse($user, 204);
    }
    else {
      throw new BadRequestHttpException("Account could not be activated."); 
    }
  }

}
